<?php

declare(strict_types=1);

namespace Drupal\multiselect_dropdown\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Checkboxes;

/**
 * Provides a form element consisting of a set of checkboxes in a dropdown.
 *
 * Here is a list of properties that are used during the rendering and form
 * processing of multiselect dropdown form elements, besides those properties
 * documented in \Drupal\Core\Render\Element\Checkboxes and
 * \Drupal\Core\Render\Element\RenderElement:
 * - #label_aria (string): An aria label added to the toggle button. Defaults
 *   to 'Toggle the list of items'.
 * - #label_none (string): Toggle button label for when no options are selected.
 *   Defaults to 'No Items Selected'.
 * - #label_all (string): Toggle button label for when all options are selected.
 *   Defaults to 'All Items'.
 * - #label_close (string): Close button label. Defaults to an empty string.
 * - #label_single (string): Toggle button label for when only one option is
 *   selected. May contain the placeholder '%d', which is replaced by the number
 *   of currently selected items. Defaults to '%d Item Selected'.
 * - #label_plural (string): Toggle button label for when more than one options
 *   are selected. May contain the placeholder '%d', which is replaced by the
 *   number of currently selected items. Defaults to '%d Items Selected'.
 * - #label_submit (string): Submit button label. Leave empty to omit a submit
 *   button within the multiselect. Defaults to an empty string.
 * - #label_clear (string): Clear button label. Leave empty to omit a clear
 *   button within the multiselect. Defaults to an empty string.
 * - #modal_breakpoint (int|string): Integer screen width at which or below the
 *   multiselect dropdown dialog should become modal, disallowing interaction
 *   with content outside the dialog until the dialog is closed, or the backed
 *   value of \Drupal\multiselect_dropdown\ModalType::Dialog or
 *   \Drupal\multiselect_dropdown\ModalType::Modal. Defaults to 768.
 * - #default_open (bool): Whether the dialog should default to open. Note that
 *   dialogs can only open on page load as non-modals, so use with caution.
 *   Defaults to false.
 *
 * Usage example:
 *
 * @code
 * $form['continents'] = [
 *   '#type' => 'multiselect_dropdown',
 *   '#title' => $this->t('Select Continents'),
 *   '#label_aria' => $this->t('Toggle the list of continents'),
 *   '#label_none' => $this->t('No Continents Selected'),
 *   '#label_all' => $this->t('All Continents'),
 *   '#label_single' => $this->t('%d Continent Selected'),
 *   '#label_plural' => $this->t('%d Continents Selected'),
 *   '#label_close' => $this->t('Close'),
 *   '#label_select_all' => $this->t('Select All'),
 *   '#label_select_none' => $this->t('Select None'),
 *   '#label_submit' => $this->t('Submit Continents'),
 *   '#label_clear' => $this->t('Clear Continents'),
 *   '#modal_breakpoint' => 768,
 *   '#default_open' => FALSE,
 *   '#options' => [
 *     'africa' => $this->t('Africa'),
 *     'asia' => $this->t('Asia'),
 *     'antarctica' => $this->t('Antarctica'),
 *     'australia' => $this->t('Australia'),
 *     'europe' => $this->t('Europe'),
 *     'north_america' => $this->t('North America'),
 *     'south_america' => $this->t('South America'),
 *   ],
 * ];
 * @endcode
 *
 * @see \Drupal\Core\Render\Element\Checkboxes
 * @see \Drupal\Core\Render\Element\RenderElement
 *
 * @FormElement("multiselect_dropdown")
 */
class MultiselectDropdown extends Checkboxes {

  /**
   * {@inheritdoc}
   */
  public function getInfo(): array {
    return [
      '#input' => TRUE,
      '#process' => [
        [static::class, 'processCheckboxes'],
      ],
      '#theme' => ['multiselect_dropdown'],
      '#theme_wrappers' => ['form_element'],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function processCheckboxes(
    &$element,
    FormStateInterface $form_state,
    &$complete_form,
  ): array {
    $element = parent::processCheckboxes($element, $form_state, $complete_form);

    if (count($element['#options'])) {
      foreach ($element['#options'] as $key => $choice) {
        $element[$key]['#attributes']['role'] = 'option';
      }
    }

    return $element;
  }

}
