<?php

declare(strict_types=1);

namespace Drupal\Tests\multiselect_dropdown\Functional;

use Drupal\multiselect_dropdown\ModalType;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\multiselect_dropdown\Traits\MultiselectDropdownTrait;

/**
 * Test multiselect dropdown form element without JavaScript.
 *
 * @covers \Drupal\multiselect_dropdown\Element\MultiselectDropdown
 * @covers multiselect_dropdown_theme
 * @covers template_preprocess_multiselect_dropdown
 *
 * @group multiselect_dropdown
 */
final class MultiselectDropdownTest extends BrowserTestBase {

  use MultiselectDropdownTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'options',
    'taxonomy',
    'multiselect_dropdown',
    'multiselect_dropdown_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  private string $path = '/multiselect-dropdown/form';

  /**
   * Dropdowns.
   *
   * @var string[]
   */
  private static array $dropdowns = [
    'colors',
    'continents',
    'open',
  ];

  /**
   * Test that the library is attached and the assets are present.
   */
  public function testAssetsLoad(): void {
    $html = $this->loadPage();
    self::assertStringContainsString('multiselect_dropdown/css/multiselect-dropdown.css', $html);
    self::assertStringContainsString('multiselect_dropdown/js/dist/multiselect-dropdowns.js', $html);
    self::assertStringContainsString('multiselect_dropdown/js/dist/multiselect-dropdown.js', $html);

    $this->container->get('module_installer')->install([
      'multiselect_dropdown_polyfill',
    ]);
    $html = $this->loadPage();
    self::assertStringContainsString('multiselect_dropdown_polyfill/css/multiselect-dropdown-polyfill.css', $html);
    self::assertStringContainsString('multiselect_dropdown_polyfill/node_modules/dialog-polyfill/dist/dialog-polyfill.css', $html);
    self::assertStringContainsString('multiselect_dropdown_polyfill/js/dist/multiselect-dropdown-polyfill.js', $html);
    self::assertStringContainsString('multiselect_dropdown_polyfill/node_modules/dialog-polyfill/dist/dialog-polyfill.esm.js', $html);
  }

  /**
   * Test that all dropdowns are present.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ElementHtmlException
   */
  public function testDropdownsPresent(): void {
    $this->loadPage();
    $session = $this->assertSession();
    foreach ($this->dropdownSelectors() as $dropdown_selector) {
      $session->elementExists('css', $dropdown_selector);
      $session->elementAttributeExists('css', $dropdown_selector, self::$attributes['dropdown']);
      $element = $session->elementAttributeExists('css', $dropdown_selector, 'id');
      self::assertNotEmpty($element->getAttribute('id'));
    }
  }

  /**
   * Test that an invalid breakpoint value throws an exception.
   */
  public function testBreakpoints(): void {
    $page = $this->drupalGet('/multiselect-dropdown/breakpoint');
    self::assertStringContainsString('ValueError', $page);
    self::assertStringContainsString(
      htmlentities('Value "breakpoint" of "#modal_breakpoint" is not allowed in multiselect dropdown "edit-breakpoint".'),
      $page,
    );

  }

  /**
   * Test that attributes generate correctly.
   *
   * @param string $dropdown_id
   * @param array<string, string|null> $attributes
   *
   * @dataProvider attributeDataProvider
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ElementHtmlException
   */
  public function testAttributes(string $dropdown_id, array $attributes): void {
    $this->loadPage();
    $session = $this->assertSession();
    $selector = $this->dropdownSelector($dropdown_id);
    foreach ($attributes as $attribute => $value) {
      if (\is_null($value)) {
        $session->elementAttributeNotExists('css', $selector, $attribute);
      }
      else {
        $session->elementAttributeContains('css', $selector, $attribute, $value);
      }
    }
  }

  /**
   * Test that labels generate correctly.
   *
   * @param string $dropdown_id
   * @param array<string, string|null> $labels
   *
   * @dataProvider buttonLabelDataProvider
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testButtonLabels(string $dropdown_id, array $labels): void {
    $this->loadPage();
    $session = $this->assertSession();
    foreach ($labels as $selector => $label) {
      $selector = $this->dropdownSelector($dropdown_id, $selector);
      if (\is_null($label)) {
        $session->elementNotExists('css', $selector);
      }
      else {
        $session->elementTextEquals('css', $selector, $label);
      }
    }
  }

  /**
   * Test that interactive elements have the needed ARIA elements.
   *
   * @throws \Behat\Mink\Exception\ElementNotFoundException
   * @throws \Behat\Mink\Exception\ElementHtmlException
   */
  public function testInteractiveElementsHaveAria(): void {
    $this->loadPage();
    $session = $this->assertSession();

    $has_aria_label = [
      'toggle',
      'close',
      'all',
      'none',
      'list',
      'submit',
      'clear',
    ];
    foreach ($has_aria_label as $aria_label) {
      $session->elementAttributeExists(
        'css',
        $this->dropdownSelector('colors', self::$selectors[$aria_label]),
        'aria-label',
      );
    }

    $toggle = $this->dropdownSelector('colors', self::$selectors['toggle']);
    $session->elementAttributeContains('css', $toggle, 'aria-expanded', 'false');
    $controls = $session
      ->elementExists('css', $toggle)
      ->getAttribute('aria-controls');
    $session->elementExists('css', "[id=\"$controls\"]");

    $list = $this->dropdownSelector('colors', self::$selectors['list']);
    $session->elementAttributeExists('css', $list, 'autofocus');
    $session->elementAttributeContains('css', $list, 'tabindex', '0');

    $session->elementAttributeContains(
      'css',
      $this->dropdownSelector('open', self::$selectors['toggle']),
      'aria-expanded',
      'true',
    );
  }

  /**
   * Test that toggle labels on page load generate correctly.
   */
  public function testToggleLabel(): void {
    $this->loadPage();
    $this->assertSession()->elementTextEquals(
      'css',
      $this->dropdownSelector('colors', self::$selectors['toggle']),
      '0 Colors Selected',
    );

    $this->loadPage(['colors' => ['red']]);
    $this->assertSession()->elementTextEquals(
      'css',
      $this->dropdownSelector('colors', self::$selectors['toggle']),
      '1 Color Selected',
    );

    $this->loadPage(['colors' => ['red', 'orange']]);
    $this->assertSession()->elementTextEquals(
      'css',
      $this->dropdownSelector('colors', self::$selectors['toggle']),
      '2 Colors Selected',
    );

    $this->loadPage(['colors' => ['red', 'orange', 'yellow', 'green', 'blue', 'purple']]);
    $this->assertSession()->elementTextEquals(
      'css',
      $this->dropdownSelector('colors', self::$selectors['toggle']),
      'All Colors Selected',
    );
  }

  /**
   * Data provider for ::testAttributes().
   *
   * @return array<string, array{id: string, attributes: array<string, string|null>}>
   */
  public static function attributeDataProvider(): array {
    $data = [];

    $data['colors'] = [
      'id' => 'colors',
      'attributes' => [
        self::$attributes['label_none'] => '%d Colors Selected',
        self::$attributes['label_all'] => 'All Colors Selected',
        self::$attributes['label_single'] => '%d Color Selected',
        self::$attributes['label_plural'] => '%d Colors Selected',
        self::$attributes['breakpoint'] => '512',
        self::$attributes['open'] => NULL,
      ],
    ];

    $data['open'] = [
      'id' => 'open',
      'attributes' => [
        self::$attributes['label_none'] => 'No Items Selected',
        self::$attributes['label_all'] => 'All Items',
        self::$attributes['label_single'] => '%d Item Selected',
        self::$attributes['label_plural'] => '%d Items Selected',
        self::$attributes['breakpoint'] => ModalType::Dialog->value,
        self::$attributes['open'] => '',
      ],
    ];

    return $data;
  }

  /**
   * Data provider for ::testButtonLabels().
   *
   * @return array<string, array{id: string, labels: array<string, string|null>}>
   */
  public static function buttonLabelDataProvider(): array {
    $data = [];

    $data['colors'] = [
      'id' => 'colors',
      'labels' => [
        self::$selectors['close'] => 'Close Colors',
        self::$selectors['all'] => 'All Colors',
        self::$selectors['none'] => 'No Colors',
        self::$selectors['submit'] => 'Submit Colors',
        self::$selectors['clear'] => 'Clear Colors',
      ],
    ];

    $data['continents'] = [
      'id' => 'continents',
      'labels' => [
        self::$selectors['close'] => '',
        self::$selectors['all'] => NULL,
        self::$selectors['none'] => NULL,
        self::$selectors['submit'] => NULL,
        self::$selectors['clear'] => NULL,
      ],
    ];

    return $data;
  }

}
